-- addon name and empty table

local a,T=...

-- create frames

local NCT=CreateFrame('frame',a)
local NCTPLA=CreateFrame('frame',a..'Player')
local NCTPET=CreateFrame('frame',a..'Pet')
local NCTP=CreateFrame('frame',a..'Panel')

-- initialize variables

NoCombatText_SV={}
local CombatFeedbackText=CombatFeedbackText
local notemp=true
local numbox=0
local temp={}
local vars={}
local defs={
	pla={dam=true,hea=true,ene=true,txt=true,},
	pet={dam=true,hea=true,ene=true,txt=true,},
}
local reg={
	pla={all,any,},
	pet={all,any,},
}

-- following function from http://lua-users.org/wiki/CopyTable
-- this copies the values of a table into another table instead of creating a reference

local function tcopy(src)
	local copy
	if type(src) == 'table' then
		copy={}
		for k,v in next,src do
			copy[tcopy(k)]=tcopy(v)
		end
		setmetatable(copy,tcopy(getmetatable(src)))
	else
		copy=src
	end
	return copy
end

-- login setup block

NCT:RegisterEvent('PLAYER_LOGIN')
NCT:SetScript('OnEvent',function()
	if NoCombatText_SV.saved then
		for k1,v1 in next,defs do
			if NoCombatText_SV[k1] == nil then
				NoCombatText_SV[k1]=defs[k1]
			end
			for k2,v2 in next,v1 do
				if NoCombatText_SV[k1][k2] == nil then
					NoCombatText_SV[k1][k2]=defs[k1][k2]
				elseif type(NoCombatText_SV[k1][k2]) ~= 'boolean' then
					NoCombatText_SV[k1][k2]=defs[k1][k2]
				end
			end
		end
		vars=tcopy(NoCombatText_SV)
		NoCombatText_SV.saved=true
	else
		vars=tcopy(defs)
		NoCombatText_SV=tcopy(defs)
		NoCombatText_SV.saved=true
	end
	NCT:Options()
	NCT:Register()
	NCT:UnregisterEvent('PLAYER_LOGIN')
end)

-- event filter blocks (if relevant text then if allowed to show then show)

local function NCTPlayer(self,event,...)
	local text='none'
	local unit,cevent,_=...
	if unit == _G['PlayerFrame'].unit then
		for k,v in next,CombatFeedbackText do if k == cevent then text=cevent end end
		if cevent == 'WOUND' then if not vars.pla.dam then CombatFeedback_OnCombatEvent(_G['PlayerFrame'],select(2,...)) end
		elseif cevent == 'HEAL' then if not vars.pla.hea then CombatFeedback_OnCombatEvent(_G['PlayerFrame'],select(2,...)) end
		elseif cevent == 'ENERGIZE' then if not vars.pla.ene then CombatFeedback_OnCombatEvent(_G['PlayerFrame'],select(2,...)) end
		elseif cevent == text then if not vars.pla.txt then CombatFeedback_OnCombatEvent(_G['PlayerFrame'],select(2,...)) end
		else CombatFeedback_OnCombatEvent(_G['PlayerFrame'],select(2,...)) end
	end
end

local function NCTPet(self,event,...)
	local text='none'
	local unit,cevent,_=...
	if unit == _G['PetFrame'].unit then
		for k,v in next,CombatFeedbackText do if k == cevent then text=cevent end end
		if cevent == 'WOUND' then if not vars.pet.dam then CombatFeedback_OnCombatEvent(_G['PetFrame'],select(2,...)) end
		elseif cevent == 'HEAL' then if not vars.pet.hea then CombatFeedback_OnCombatEvent(_G['PetFrame'],select(2,...)) end
		elseif cevent == 'ENERGIZE' then if not vars.pet.ene then CombatFeedback_OnCombatEvent(_G['PetFrame'],select(2,...)) end
		elseif cevent == text then if not vars.pet.txt then CombatFeedback_OnCombatEvent(_G['PetFrame'],select(2,...)) end
		else CombatFeedback_OnCombatEvent(_G['PetFrame'],select(2,...)) end
	end
end

-- event registration block

function NCT:Register()
	reg.pla.all=true
	reg.pet.all=true
	reg.pla.any=false
	reg.pet.any=false
	for k,v in next,vars.pla do
		if v then reg.pla.any=true
		else reg.pla.all=false end
	end
	for k,v in next,vars.pet do
		if v then reg.pet.any=true
		else reg.pet.all=false end
	end
	if reg.pla.all then
		PlayerFrame:UnregisterEvent('UNIT_COMBAT')
		NCTPLA:UnregisterEvent('UNIT_COMBAT')
		NCTPLA:SetScript('OnEvent',nil)
	elseif reg.pla.any then
		PlayerFrame:UnregisterEvent('UNIT_COMBAT')
		NCTPLA:RegisterUnitEvent('UNIT_COMBAT','player','vehicle')
		NCTPLA:SetScript('OnEvent',NCTPlayer)
	else
		PlayerFrame:RegisterUnitEvent('UNIT_COMBAT','player','vehicle')
		NCTPLA:UnregisterEvent('UNIT_COMBAT')
		NCTPLA:SetScript('OnEvent',nil)
	end
	if reg.pet.all then
		PetFrame:UnregisterEvent('UNIT_COMBAT')
		NCTPET:UnregisterEvent('UNIT_COMBAT')
		NCTPET:SetScript('OnEvent',nil)
	elseif reg.pet.any then
		PetFrame:UnregisterEvent('UNIT_COMBAT')
		NCTPET:RegisterUnitEvent('UNIT_COMBAT','pet','player')
		NCTPET:SetScript('OnEvent',NCTPet)
	else
		PetFrame:RegisterUnitEvent('UNIT_COMBAT','pet','player')
		NCTPET:UnregisterEvent('UNIT_COMBAT')
		NCTPET:SetScript('OnEvent',nil)
	end
end

-- interface panel

function NCT:CreateBox(xloc,yloc,text)
	numbox=numbox+1
	local name=a..'CheckBox'..format('%02i',numbox)
	local box=CreateFrame('CheckButton',name,NCTP,'InterfaceOptionsCheckButtonTemplate')
	box:SetPoint('TOPLEFT',NCTP,'TOPLEFT',xloc,yloc)
	_G[name..'Text']:SetText('|cffffd100'..strlower(text)..'|cffffd100)')
	return box
end

function NCT:Options()
	NCTP.name=a
	InterfaceOptions_AddCategory(NCTP,a)
	NCTTitle=NCTP:CreateFontString('$parentTitle','ARTWORK','GameFontNormalLarge')
	NCTTitle:SetPoint('TOPLEFT',NCTP,'TOPLEFT',16,-16)
	NCTTitle:SetText(DISABLE)
	NCTCheckBox01=NCT:CreateBox(16,-64,''..PLAYER..' '..DAMAGE..' '..EVENTS_LABEL..' (|cffffff002413|cffffd100/|cffffffff2413')
	NCTCheckBox02=NCT:CreateBox(16,-96,''..PLAYER..' '..VOICEMACRO_LABEL_HEALME1..' '..EVENTS_LABEL..' (|cff00ff002413')
	NCTCheckBox03=NCT:CreateBox(16,-128,''..PLAYER..' '..COMBAT_TEXT_SHOW_ENERGIZE_TEXT..' (|cff69ccf02413')
	NCTCheckBox04=NCT:CreateBox(16,-160,''..PLAYER..' '..COMBAT..' '..EVENTS_LABEL..' (|cffffffff'..COMBAT_TEXT_SHOW_DODGE_PARRY_MISS_TEXT..'')
	NCTCheckBox05=NCT:CreateBox(16,-192,''..PET..' '..DAMAGE..' '..EVENTS_LABEL..' (|cffffff002413|cffffd100/|cffffffff2413')
	NCTCheckBox06=NCT:CreateBox(16,-224,''..PET..' '..VOICEMACRO_LABEL_HEALME1..' '..EVENTS_LABEL..' (|cff00ff002413')
	NCTCheckBox07=NCT:CreateBox(16,-256,''..PET..' '..COMBAT_TEXT_SHOW_ENERGIZE_TEXT..' (|cff69ccf02413')
	NCTCheckBox08=NCT:CreateBox(16,-288,''..PET..' '..COMBAT..' '..EVENTS_LABEL..' (|cffffffff'..COMBAT_TEXT_SHOW_DODGE_PARRY_MISS_TEXT..'')
	NCTCheckBox01:SetScript('PostClick',function()vars.pla.dam=NCTCheckBox01:GetChecked()NCT:Register()end)
	NCTCheckBox02:SetScript('PostClick',function()vars.pla.hea=NCTCheckBox02:GetChecked()NCT:Register()end)
	NCTCheckBox03:SetScript('PostClick',function()vars.pla.ene=NCTCheckBox03:GetChecked()NCT:Register()end)
	NCTCheckBox04:SetScript('PostClick',function()vars.pla.txt=NCTCheckBox04:GetChecked()NCT:Register()end)
	NCTCheckBox05:SetScript('PostClick',function()vars.pet.dam=NCTCheckBox05:GetChecked()NCT:Register()end)
	NCTCheckBox06:SetScript('PostClick',function()vars.pet.hea=NCTCheckBox06:GetChecked()NCT:Register()end)
	NCTCheckBox07:SetScript('PostClick',function()vars.pet.ene=NCTCheckBox07:GetChecked()NCT:Register()end)
	NCTCheckBox08:SetScript('PostClick',function()vars.pet.txt=NCTCheckBox08:GetChecked()NCT:Register()end)
	NCTP.okay=function()
		NoCombatText_SV=tcopy(vars)
		notemp=true
	end
	NCTP.cancel=function()
		vars=tcopy(temp)
		notemp=true
		NCT:Register()
	end
	NCTP.default=function()
		vars=tcopy(defs)
		NoCombatText_SV=tcopy(defs)
		NCT:SetCheckBoxes()
		NCT:Register()
	end
	NCTP.refresh=function()
		NCT:SetCheckBoxes()
		if notemp then temp=tcopy(vars) notemp=false end
	end
	SLASH_NOCOMBATTEXT_1='/nocombattext'
	SLASH_NOCOMBATTEXT_2='/nct'
	SlashCmdList['NOCOMBATTEXT_']=function()InterfaceOptionsFrame_OpenToCategory(a)end
end

function NCT:SetCheckBoxes()
	NCTCheckBox01:SetChecked(vars.pla.dam)
	NCTCheckBox02:SetChecked(vars.pla.hea)
	NCTCheckBox03:SetChecked(vars.pla.ene)
	NCTCheckBox04:SetChecked(vars.pla.txt)
	NCTCheckBox05:SetChecked(vars.pet.dam)
	NCTCheckBox06:SetChecked(vars.pet.hea)
	NCTCheckBox07:SetChecked(vars.pet.ene)
	NCTCheckBox08:SetChecked(vars.pet.txt)
end